@@ -13,6 +13,7 @@ import sys
 import math
 import copy
 import pdfstitcher.utils as utils
+from pdfstitcher.utils import Config
 
 
 class SW_ROTATION(IntEnum):
@@ -53,9 +54,7 @@ class PageTiler:
         target_height=None,
         vertical_align=SW_ALIGN_V.BOTTOM,
         horizontal_align=SW_ALIGN_H.LEFT,
-        units=utils.UNITS.INCHES,
     ):
-
         self.in_doc = in_doc
 
         if isinstance(page_range, str):
@@ -65,7 +64,7 @@ class PageTiler:
         else:
             self.page_range = []
 
-        self.set_trim(trim)
+        self.trim = trim
         self.margin = margin
         self.rotation = rotation
         self.actually_trim = actually_trim
@@ -82,29 +81,30 @@ class PageTiler:
         self.target_height = target_height
         self.target_width = target_width
 
-        self.units = units
         if self.target_height:
-            self.target_height = self.units.units_to_px(self.target_height)
+            self.target_height = Config.general["units"].units_to_px(self.target_height)
         if self.target_width:
-            self.target_width = self.units.units_to_px(self.target_width)
+            self.target_width = Config.general["units"].units_to_px(self.target_width)
 
         # Optional vertical and horizontal alignment in case pages are non-uniform in size
         self.vertical_align = vertical_align
         self.horizontal_align = horizontal_align
 
-    def set_trim(self, trim):
+    @property
+    def trim(self):
+        return self._trim
+
+    @trim.setter
+    def trim(self, trim):
         if len(trim) == 1:
-            self.trim = [trim, trim, trim, trim]
-
-        if len(trim) == 2:
-            self.trim = [trim[0], trim[0], trim[1], trim[1]]
-
-        if len(trim) == 4:
-            self.trim = trim
-
+            self._trim = [trim[0], trim[0], trim[0], trim[0]]
+        elif len(trim) == 2:
+            self._trim = [trim[0], trim[0], trim[1], trim[1]]
+        elif len(trim) == 4:
+            self._trim = trim
         else:
             print(_("Invalid trim value specified, ignoring"))
-            self.trim = [0, 0, 0, 0]
+            self._trim = [0, 0, 0, 0]
 
     def show_options(self):
         # convert the margin and trim options into pixels
@@ -147,61 +147,26 @@ class PageTiler:
 
         print(_("Tiling with {} rows and {} columns").format(self.rows, self.cols))
         print(_("Options") + ":")
-        print("    " + _("Margins") + ": {} {}".format(self.margin, self.units.str))
-        print("    " + _("Trim") + ": {} {}".format(self.trim, self.units.str))
+        print("    " + _("Margins") + ": {} {}".format(self.margin, Config.general["units"]))
+        print("    " + _("Trim") + ": {} {}".format(self.trim, Config.general["units"]))
         print("    " + _("Rotation") + ": {}".format(rotstr))
         print("    " + _("Page order") + ": {}, {}, {}".format(orderstr, lrstr, btstr))
         print("    " + _("Vertical alignment") + ": {}".format(alvstr))
         print("    " + _("Horizontal alignment") + ": {}".format(alhstr))
 
-    def run(
-        self,
-        rows=None,
-        cols=None,
-        target_width=None,
-        target_height=None,
-        vertical_align=None,
-        horizontal_align=None,
-    ):
+    def build_pagelist(self, new_doc: pikepdf.Pdf, trim: list) -> tuple:
         """
-        The main function for tiling pages.
+        Loops through the pages and constructs the list of pages, their length/width, and XObjects.
+        Returns a tuple containing content_dict, pw, ph, page_names.
+        Still too much going on in this function!
         """
-
-        # First, go through the pages and normalize the various boxes
-        for page in self.in_doc.pages:
-            utils.normalize_boxes(page)
-
-        if rows or cols:
-            self.rows = rows
-            self.cols = cols
-
-        if target_width is not None:
-            self.target_width = self.units.units_to_px(target_width)
-        if target_height is not None:
-            self.target_height = self.units.units_to_px(target_height)
-
-        if vertical_align is not None:
-            self.vertical_align = vertical_align
-        if horizontal_align is not None:
-            self.horizontal_align = horizontal_align
-
-        if self.in_doc is None:
-            print(_("Input document not loaded"))
-            return
-
-        # initialize a new document
-        new_doc = utils.init_new_doc(self.in_doc)
-
-        # define the dictionary to store xobjects
+        # define the dictionary to store xobjects and the corresponding names (e.g. MC0, MC1, etc.)
         content_dict = pikepdf.Dictionary({})
-
         page_names = []
         pw = []
         ph = []
 
         page_count = len(self.in_doc.pages)
-        trim = [self.units.units_to_px(t) for t in self.trim]
-
         # initialize the width/height indices based on page rotation
         page_rot = 0
 
@@ -216,10 +181,6 @@ class PageTiler:
         ref_page = self.in_doc.pages[p - 1]
         ref_width, ref_height = utils.get_page_dims(ref_page, page_rot)
 
-        user_unit = 1
-        if "/UserUnit" in ref_page.keys():
-            user_unit = float(ref_page.UserUnit)
-
         different_size = set()
 
         for p in self.page_range:
@@ -276,6 +237,7 @@ class PageTiler:
                 ref_width = p_width
                 ref_height = p_height
 
+                # magic sauce to copy the info to the new document as an XOBject
                 content_dict[pagekey] = new_doc.copy_foreign(new_page.as_form_xobject())
 
             else:
@@ -291,9 +253,63 @@ class PageTiler:
                 )
             )
 
-        n_tiles = len(page_names)
+        return content_dict, pw, ph, page_names
 
-        # check which one is specified
+    def adjust_trim_order(self, px_trim: list) -> None:
+        """
+        Rearranges the trim order based on page rotation.
+        """
+        # swap the trim order
+        # default: left,right,top,bottom
+        order = [0, 1, 2, 3]
+
+        if self.rotation == SW_ROTATION.CLOCKWISE:
+            order = [3, 2, 0, 1]
+        if self.rotation == SW_ROTATION.COUNTERCLOCKWISE:
+            order = [2, 3, 1, 0]
+        if self.rotation == SW_ROTATION.TURNAROUND:
+            order = [1, 0, 3, 2]
+
+        px_trim = [px_trim[o] for o in order]
+
+    def compute_target_size(self, n_tiles: int, pw: list, ph: list, trim: list) -> tuple:
+        """
+        Find the grid that contains the maximum page size for each row/col.
+        Returns the grid dimensions as a tuple of two lists.
+        """
+        col_width = [0] * self.cols
+        row_height = [0] * self.rows
+
+        if self.col_major:
+            for c in range(self.cols):
+                col_width[c] = max(pw[c * self.rows : c * self.rows + self.rows]) - (
+                    trim[0] + trim[1]
+                )
+
+            for r in range(self.rows):
+                row_height[r] = max(ph[r : n_tiles : self.cols]) - (trim[2] + trim[3])
+        else:
+            for r in range(self.rows):
+                row_height[r] = max(ph[r * self.cols : r * self.cols + self.cols]) - (
+                    trim[2] + trim[3]
+                )
+
+            for c in range(self.cols):
+                col_width[c] = max(pw[c : n_tiles : self.rows]) - (trim[0] + trim[1])
+
+        if self.right_to_left:
+            col_width.reverse()
+
+        if self.bottom_to_top:
+            row_height.reverse()
+
+        return col_width, row_height
+
+    def calc_rows_cols(self, n_tiles: int) -> bool:
+        """
+        Calculate the number of rows/columns requested based on the number of pages to tile.
+        Returns True if the result is valid, False otherwise.
+        """
         if self.cols is not None and self.cols > 0:
             self.rows = math.ceil(n_tiles / self.cols)
             if self.rows == 1 and self.cols > n_tiles:
@@ -318,32 +334,81 @@ class PageTiler:
             self.cols = math.ceil(math.sqrt(n_tiles))
             self.rows = math.ceil(n_tiles / self.cols)
 
+        # Make sure there are no empty columns or rows
+        if self.col_major:
+            return self.cols * self.rows - n_tiles < self.rows
+        else:
+            return self.cols * self.rows - n_tiles < self.cols
+
+    def run(
+        self,
+        rows=None,
+        cols=None,
+        target_width=None,
+        target_height=None,
+        vertical_align=None,
+        horizontal_align=None,
+    ):
+        """
+        The main function for tiling pages.
+        """
+
+        if self.in_doc is None:
+            print(_("Input document not loaded"))
+            return
+
+        # First, go through the pages and normalize the various boxes
+        for page in self.in_doc.pages:
+            utils.normalize_boxes(page)
+
+        if rows or cols:
+            self.rows = rows
+            self.cols = cols
+
+        if target_width is not None:
+            self.target_width = Config.general["units"].units_to_px(target_width)
+        if target_height is not None:
+            self.target_height = Config.general["units"].units_to_px(target_height)
+
+        if vertical_align is not None:
+            self.vertical_align = vertical_align
+        if horizontal_align is not None:
+            self.horizontal_align = horizontal_align
+
+        # initialize a new document
+        new_doc = utils.init_new_doc(self.in_doc)
+
+        # get the user unit from the first page (either 1 or 10, if it's a huge page)
+        user_unit = 1
+        first_page = self.in_doc.pages[self.page_range[0] - 1]
+        if "/UserUnit" in first_page.keys():
+            user_unit = float(first_page.UserUnit)
+
+        # define the trim in pdf units, then build the page list
+        px_trim = [Config.general["units"].units_to_px(t / user_unit) for t in self.trim]
+        content_dict, pw, ph, page_names = self.build_pagelist(new_doc, px_trim)
+        n_tiles = len(page_names)
+        if not self.calc_rows_cols(n_tiles):
+            error_msg = _(
+                "Error: cannot tile {} pages with {} rows and {} columns".format(
+                    n_tiles, self.rows, self.cols
+                )
+            )
+            if self.col_major:
+                error_msg += "\n" + _("filling columns first, the last column would be empty.")
+            else:
+                error_msg += "\n" + _("filling rows first, the last row would be empty.")
+            print(error_msg)
+            return
+
         # after calculating rows/cols but before reordering trim, show the user the selected options
         self.show_options()
-
-        # swap the trim order
-        # default: left,right,top,bottom
-        order = [0, 1, 2, 3]
-
-        if self.rotation == SW_ROTATION.CLOCKWISE:
-            order = [3, 2, 0, 1]
-        if self.rotation == SW_ROTATION.COUNTERCLOCKWISE:
-            order = [2, 3, 1, 0]
-        if self.rotation == SW_ROTATION.TURNAROUND:
-            order = [1, 0, 3, 2]
-
-        trim = [self.units.units_to_px(t / user_unit) for t in self.trim]
-        trim = [trim[o] for o in order]
+        self.adjust_trim_order(px_trim)
 
         if self.rotation in (SW_ROTATION.CLOCKWISE, SW_ROTATION.COUNTERCLOCKWISE):
             # swap width and height of pages
             ph, pw = pw, ph
 
-        width = 0
-        height = 0
-        col_width = [0] * self.cols
-        row_height = [0] * self.rows
-
         if self.target_width and self.target_height:
             # determine size of each page based on requested dimensions
             width = self.target_width
@@ -352,45 +417,23 @@ class PageTiler:
             page_box_height = self.target_height / self.rows
             page_box_defined = True
         else:
-            # Find the grid that contains the maximum page size for each row/col
-            if self.col_major:
-                for c in range(self.cols):
-                    col_width[c] = max(pw[c * self.rows : c * self.rows + self.rows]) - (
-                        trim[0] + trim[1]
-                    )
-
-                for r in range(self.rows):
-                    row_height[r] = max(ph[r : n_tiles : self.cols]) - (trim[2] + trim[3])
-            else:
-                for r in range(self.rows):
-                    row_height[r] = max(ph[r * self.cols : r * self.cols + self.cols]) - (
-                        trim[2] + trim[3]
-                    )
-
-                for c in range(self.cols):
-                    col_width[c] = max(pw[c : n_tiles : self.rows]) - (trim[0] + trim[1])
-
-            if self.right_to_left:
-                col_width.reverse()
-
-            if self.bottom_to_top:
-                row_height.reverse()
-
+            col_width, row_height = self.compute_target_size(n_tiles, pw, ph, px_trim)
             width = sum(col_width)
             height = sum(row_height)
             page_box_defined = False
 
         # create a new document with a page big enough to contain all the tiled pages, plus requested margin
-        margin = self.units.units_to_px(self.margin / user_unit)
+        margin = Config.general["units"].units_to_px(self.margin / user_unit)
         media_box = [
-            float(new_page.MediaBox[0]),
-            float(new_page.MediaBox[1]),
+            float(first_page.MediaBox[0]),
+            float(first_page.MediaBox[1]),
             width + 2 * margin,
             height + 2 * margin,
         ]
 
         utils.print_media_box(media_box)
 
+        # TODO: Refactor this giant loop into two functions (scale to fit and no scaling)
         i = 0
         content_txt = ""
         performed_scale = False
@@ -423,13 +466,13 @@ class PageTiler:
                 scalef_height = page_box_height / source_height
                 # take the smaller scaling factor so that the page will fit into its box
                 scale_factor = min(scalef_width, scalef_height)
-                cpos_x0 = c * page_box_width - c * (trim[0] + trim[1])
+                cpos_x0 = c * page_box_width - c * (px_trim[0] + px_trim[1])
                 cpos_y0 = (self.rows - r - 1) * page_box_height - (self.rows - r - 1) * (
-                    trim[2] + trim[3]
+                    px_trim[2] + px_trim[3]
                 )
             else:
-                cpos_x0 = sum(col_width[:c]) - trim[0]
-                cpos_y0 = sum(row_height[r + 1 :]) - trim[3]
+                cpos_x0 = sum(col_width[:c]) - px_trim[0]
+                cpos_y0 = sum(row_height[r + 1 :]) - px_trim[3]
 
                 # store the page box height/width for convenience if rotation is needed
                 page_box_height = ph[i]
@@ -525,7 +568,6 @@ class PageTiler:
 
 
 def main(args):
-
     utils.setup_locale()
 
     # first try opening the document
@@ -542,28 +584,25 @@ def main(args):
     else:
         tiler.page_range = [i + 1 for i in range(len(in_doc.pages))]
 
+    if args.units == "cm":
+        Config.general["units"] = utils.UNITS.CENTIMETERS
+
     if args.margin is not None:
-        # all the docs/examples seem to assume 72 dpi all the time
         tiler.margin = utils.txt_to_float(args.margin)
 
     if args.trim is not None:
         trim = [utils.txt_to_float(t) for t in args.trim.split(",")]
-        tiler.set_trim(trim)
+        tiler.trim = trim
 
-    if args.rotate is not None:
-        failed = False
-        r = int(args.rotate)
-        if r == 0:
-            tiler.rotation = SW_ROTATION.NONE
-        elif r == 90:
-            tiler.rotation = SW_ROTATION.CLOCKWISE
-        elif r == 180:
-            tiler.rotation = SW_ROTATION.TURNAROUND
-        elif r == 270:
-            tiler.rotation = SW_ROTATION.COUNTERCLOCKWISE
-        else:
-            print(_("Invalid rotation value"))
-            sys.exit()
+    r = int(args.rotate)
+    if r == 0:
+        tiler.rotation = SW_ROTATION.NONE
+    elif r == 90:
+        tiler.rotation = SW_ROTATION.CLOCKWISE
+    elif r == 180:
+        tiler.rotation = SW_ROTATION.TURNAROUND
+    elif r == 270:
+        tiler.rotation = SW_ROTATION.COUNTERCLOCKWISE
 
     tiler.cols = 0
     tiler.rows = 0
@@ -620,32 +659,44 @@ def parse_arguments():
         type=int,
         help=_("Number of columns in tiled grid."),
     )
+    parser.add_argument(
+        "-u",
+        "--units",
+        choices=["in", "cm"],
+        default="in",
+        help=_("Units for margin and trim values."),
+    )
     parser.add_argument(
         "-m",
         "--margin",
-        help=_("Margin size in inches."),
+        help=_("Margin size in selected units."),
     )
     parser.add_argument(
         "-t",
         "--trim",
-        help=_("Amount to trim from edges")
+        help=_("Amount to trim from edges in selected units")
         + " "
-        + _(
-            "given as left,right,top,bottom (e.g. 0.5,0,0.5,0 would trim half an inch from left and top)"
-        ),
+        + _("given as left,right,top,bottom (e.g. 0.5,0,0.5,0 would trim 0.5 from left and top)"),
     )
     parser.add_argument(
         "-R",
         "--rotate",
         type=int,
-        help=_("Rotate pages (90, 180, or 270 degrees)"),
+        default=0,
+        choices=[0, 90, 180, 270],
+        help=_("Rotate pages"),
+    )
+    parser.add_argument(
+        "--col-major",
+        type=bool,
+        default=False,
+        help=_("Fill columns before rows (default is rows first)"),
     )
 
     return parser.parse_args()
 
 
 if __name__ == "__main__":
-
     utils.setup_locale()
     args = parse_arguments()
     new_doc, success = main(args)
