@@ -183,22 +183,15 @@ class PDFStitcherFrame(wx.Frame):
         # do it
         try:
             if do_layers:
-                self.progress_win = wx.ProgressDialog(
-                    "Processing layers",
-                    "Processing layers, please wait",
-                    style=wx.PD_CAN_ABORT,
-                )
-                filtered = self.layer_filter.run(
-                    self.set_progress_range,
-                    self.update_progress,
-                    self.progress_was_cancelled,
+                progress_win = wx.ProgressDialog(
+                    _("Processing layers"),
+                    _("Processing layers, please wait"),
+                    style=wx.PD_CAN_ABORT | wx.PD_AUTO_HIDE,
                 )
+                filtered = self.layer_filter.run(progress_win)
             else:
                 filtered = self.in_doc
 
-            if self.progress_win:
-                self.update_progress(self.progress_win.GetRange())
-
             if filtered is None:
                 return
 
@@ -230,24 +223,6 @@ class PDFStitcherFrame(wx.Frame):
             print(e)
             print(_("Make sure " + self.out_doc_path + " isn't open in another program"))
 
-    def progress_was_cancelled(self):
-        if self.progress_win != None:
-            return self.progress_win.WasCancelled()
-
-    def update_progress(self, val):
-        if self.progress_win != None:
-            p_range = self.progress_win.GetRange()
-            if p_range:
-                if val == p_range:
-                    # Hit the max, we're done
-                    self.progress_win = None
-                else:
-                    self.progress_win.Update(val)
-
-    def set_progress_range(self, val):
-        if self.progress_win != None:
-            self.progress_win.SetRange(val)
-
     def make_menu_bar(self):
         """
         Make and populate the menubar.
