@@ -9,13 +9,24 @@ import sys
 import importlib.metadata
 import requests
 from pdfstitcher import utils
+import os
+import packaging
 
 __version__ = importlib.metadata.version("pdfstitcher")
 
 
-def update_available():
+def is_flatpak() -> bool:
+    """
+    Returns True if the app is running in a Flatpak sandbox.
+    """
+    return "FLATPAK_ID" in os.environ
+
+
+def update_available() -> str:
     """
     Checks whether there's a new release of PDFStitcher.
+    Returns a string containing the latest version if there's a new one,
+    or None if the current version is up to date.
     """
     response = requests.get(utils.PYPI_HOME + "/json")
     if not response.ok:
@@ -23,21 +34,13 @@ def update_available():
 
     pypi_version = response.json()["info"]["version"]
 
-    current = [int(num) for num in __version__.split(".")]
-    pypi = [int(num) for num in pypi_version.split(".")]
-
-    if len(current) == 2:
-        current.append(0)
-    if len(pypi) == 2:
-        pypi.append(0)
-
-    if any(pypi[i] > current[i] for i in range(3)):
+    if packaging.version.parse(pypi_version) > packaging.version.parse(__version__):
         return pypi_version
     else:
         return None
 
 
-def get_download_url():
+def get_download_url() -> str:
     """
     Returns the platform-specific download link for the latest release.
     """
